# The version and release of the driver as specified by AMD.
%define basever 16.30.3
%define baserel 315407

# Version of X11 server and the corresponding X11 Video ABI the driver
# should support.
# 
# [IMPORTANT] Update these when updating the X11 server.
%define x11ver 1.18
%define videodrv_major 20

# Priority for the alternatives. Any number higher than 500 (the default
# priority the X11 server uses) would do.
%define priority 1016

%ifarch x86_64
%define pkg_ver_suffix %{basever}-%{baserel}_amd64
%else
%define pkg_ver_suffix %{basever}-%{baserel}_i386
%endif

# Paths
%define amdgpu_libdir %{_libdir}/%{name}
%define amdgpu_includedir %{_includedir}/%{name}

%define xorg_driverdir %{_libdir}/xorg/modules/drivers
%define xorg_extra_modules %{_libdir}/xorg/extra-modules
%define xorg_confdir %{_datadir}/X11/xorg.conf.d
%define amdgpu_extdir %{amdgpu_libdir}/xorg

%ifarch x86_64
%define deb_libpath /usr/lib/x86_64-linux-gnu
%else
%define deb_libpath /usr/lib/i386-linux-gnu
%endif

%define pkg_xorg_modulesdir %{deb_libpath}/%{name}/%{x11ver}/modules

# --------------------------------------------------------------------------

# Disable debug rpms.
%define _enable_debug_packages	%{nil}
%define debug_package		%{nil}

# FIXME: fix the cases when immediate symbol resolution is not enough.
# For the present, fall back to the old symbol resolution rules for ld.
%define _disable_ld_now 1

# --------------------------------------------------------------------------

# Do not provide libGL.so.1 and the like. Unrelated packages should not
# pull in any AMDGPU PRO package when they just need libGL.so.1, etc.
%define __noautoprov '(.*)\\.so(.*)|devel\\(libOpenCL(.*)'

# Do not create requirements for the OpenCL libraries automatically, there
# are no such provides dues to __noautoprov above.
# The respective packages should be listed in "Requires" explicitly.
%define __noautoreq 'libOpenCL\\.so(.*)'

# --------------------------------------------------------------------------

# Library names & versions

%define drm_major 2
%define libdrm %mklibname %{name}_drm %{drm_major}
%define libdrm_dev %mklibname %{name}_drm -d

%define kms_major 1
%define libkms %mklibname %{name}_kms %{kms_major}

%define drm_amdgpu_major 1
%define libdrm_amdgpu %mklibname %{name}_drm_amdgpu %{drm_amdgpu_major}

%define egl_major 1
%define libegl %mklibname %{name}_egl %{egl_major}
%define libegl_dev %mklibname %{name}_egl -d

%define gbm_major 1
%define libgbm %mklibname %{name}_gbm %{gbm_major}
%define libgbm_dev %mklibname %{name}_gbm -d

%define libdri %mklibname %{name}_dri

#define vulkan_major 1
%define libvulkan %mklibname %{name}_vulkan

# From *_glx.*.deb
%define gl_major 1
%define libgl %mklibname %{name}_gl %{gl_major}
%define libgl_dev %mklibname %{name}_gl -d

%define gles_major 2
%define libgles %mklibname %{name}_gles %{gles_major}
%define libgles_dev %mklibname %{name}_gles -d

%define vdpau_major 1
%define libvdpau %mklibname %{name}_vdpau %{vdpau_major}

%define opencl_major 1
%define libopencl %mklibname %{name}_opencl %{opencl_major}
%define libopencl_dev %mklibname %{name}_opencl -d

# --------------------------------------------------------------------------

Summary:	AMDGPU PRO graphics drivers provided by AMD
Name:		amdgpu-pro
Version:	%{basever}.%{baserel}
Release:	4.beta
License:	Freeware
URL:		http://support.amd.com/en-us/kb-articles/Pages/AMD-Radeon-GPU-PRO-Linux-Beta-Driver%%E2%%80%%93Release-Notes.aspx
Group:		System/Kernel and hardware

# The driver itself is currently for x86_64 only. But this project also
# provides the 32-bit libraries to run 32-bit apps with AMDGPU PRO on
# 64-bit systems. These packages with these libraries are kept in the usual
# repositories for 32-bit packages.
ExclusiveArch:	%{ix86} x86_64

Source0:	https://www2.ati.com/drivers/linux/%{name}_%{basever}-%{baserel}.tar.xz
Source100:	%{name}.rpmlintrc

%ifarch x86_64
Provides:	should-restart = system

Requires:	%{name}-graphics = %{EVRD}
Requires:	%{name}-computing = %{EVRD}

# AMDGPU PRO might work OK with some older kernel versions too, but let us
# not bother with those.
Requires:	kernel >= 4.8

%else
# For convenience, just in case someone installs both 64-bit and 32-bit
# "amdgpu-pro" packages.
Requires:	%{name}-lib32 = %{EVRD}
%endif

%description
This package pulls in the commonly used components of AMDGPU PRO. If you are
not sure which packages to install to get AMDGPU PRO, install this one.

%files
%ifarch x86_64
%dir %{_datadir}/doc/%{name}
%doc %{_datadir}/doc/%{name}/copyright
%endif

# --------------------------------------------------------------------------

%package -n	%{libdrm}
Summary:	Userspace interface to kernel DRM services
Group:		System/Libraries

%description -n	%{libdrm}
Userspace interface to kernel DRM services.

%files -n	%{libdrm}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libdrm.so*

%post -n	%{libdrm}
# Make sure the system will find this library. amdgpu-pro-core sets the
# search paths appropriately but we still need to run ldconfig here.
/sbin/ldconfig -X

%postun -n	%{libdrm}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libkms}
Summary:	Shared library for KMS
Group:		System/Libraries
Requires:	%{libdrm} = %{EVRD}

%description -n	%{libkms}
Shared library for kernel mode setting.

%files -n	%{libkms}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libkms.so*

%post -n	%{libkms}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libkms}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libdrm_amdgpu}
Summary:	Userspace interface to AMDGPU-specific kernel DRM services
Group:		System/Libraries
Requires:	%{libdrm} = %{EVRD}
Requires:	%{libkms} = %{EVRD}

%description -n	%{libdrm_amdgpu}
Userspace interface to AMDGPU-specific kernel DRM services

%files -n	%{libdrm_amdgpu}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libdrm_amdgpu.so*

%post -n	%{libdrm_amdgpu}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libdrm_amdgpu}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libegl}
Summary:	Implementation of the EGL API - runtime
Group:		System/Libraries

%description -n	%{libegl}
Implementation of the EGL API - runtime.

%files -n	%{libegl}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libEGL.so*

%post -n	%{libegl}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libegl}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libgbm}
Summary:	Generic buffer management API - runtime
Group:		System/Libraries
Requires:	%{libdrm} = %{EVRD}
Requires:	%{libdrm_amdgpu} = %{EVRD}

%description -n	%{libgbm}
Generic buffer management API - runtime

%files -n	%{libgbm}
%dir %{amdgpu_libdir}/gbm
%{amdgpu_libdir}/libgbm.so*
%{amdgpu_libdir}/gbm/*

%post -n	%{libgbm}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libgbm}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libgl}
Summary:	Implementation of the OpenGL API - GLX runtime
Group:		System/Libraries
Requires:	%{libdrm_amdgpu} = %{EVRD}

%description -n	%{libgl}
Implementation of the OpenGL API - GLX runtime.

%files -n	%{libgl}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libGL.so*

%post -n	%{libgl}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libgl}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libgles}
Summary:	Implementation of the OpenGL|ES 2.x API - runtime
Group:		System/Libraries
Requires:	%{libegl} = %{EVRD}

%description -n	%{libgles}
Implementation of the OpenGL|ES 2.x API - runtime.

%files -n	%{libgles}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libGLESv2.so*

%post -n	%{libgles}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libgles}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libvdpau}
Summary:	AMDGPU Pro VDPAU driver
Group:		System/X11
Requires:	%{libdrm} = %{EVRD}
Requires:	%{libdrm_amdgpu} = %{EVRD}

%description -n	%{libvdpau}
AMDGPU Pro VDPAU driver.

%files -n	%{libvdpau}
%{_libdir}/vdpau/*

# --------------------------------------------------------------------------

# OpenCL
%package -n	%{libopencl}
Summary:	AMD OpenCL ICD Loader library
Group:		System/Libraries

%description -n	%{libopencl}
AMD OpenCL ICD Loader library.

%files -n	%{libopencl}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libOpenCL.so.%{opencl_major}

%post -n	%{libopencl}
# See the notes for libdrm above.
/sbin/ldconfig -X

%postun -n	%{libopencl}
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package -n	%{libopencl_dev}
Summary:	Development files for %{libopencl}
Group:		System/Libraries

Requires:	%{libopencl} = %{EVRD}

%description -n	%{libopencl_dev}
Development files for %{libopencl}.

%files -n 	%{libopencl_dev}
%dir %{amdgpu_libdir}
%{amdgpu_libdir}/libOpenCL.so
%{_libdir}/libOpenCL.so

# --------------------------------------------------------------------------

# DRI
%package -n	%{libdri}
Summary:	Implementation of the OpenGL API - DRI modules
Group:		System/Libraries

%description -n	%{libdri}
Implementation of the OpenGL API - DRI modules.

%files -n	%{libdri}
%dir %{_libdir}/dri
%{_libdir}/dri/*.so*
%{deb_libpath}/dri

# --------------------------------------------------------------------------

# amdgpu-pro-opencl-icd
%package	opencl-icd
Summary:	Non-free AMD OpenCL ICD Loaders
Group:		System/X11

%description	opencl-icd
Non-free AMD OpenCL ICD Loaders

%files		opencl-icd
%dir %{amdgpu_libdir}
%{_sysconfdir}/OpenCL/vendors/*.icd
%{amdgpu_libdir}/libamdocl*.so*

# --------------------------------------------------------------------------

# vulkan driver
%package -n	%{libvulkan}
Summary:	AMDGPU PRO Vulkan driver
Group:		System/X11

Requires:	%{libdrm_amdgpu} = %{EVRD}
Provides:	%{name}-vulkan-driver = %{EVRD}

%description -n	%{libvulkan}
AMDGPU PRO Vulkan driver.

%files -n	%{libvulkan}
%dir %{_sysconfdir}/vulkan/icd.d
%{_sysconfdir}/vulkan/icd.d/amd_icd*.json
%{_libdir}/amdvlk*.so

# --------------------------------------------------------------------------

%ifarch x86_64

%package	clinfo
Summary:	AMD OpenCL info utility
Group:		System/X11

Requires:	%{libopencl} = %{EVRD}

%description	clinfo
AMD OpenCL info utility

%files		clinfo
%{_bindir}/clinfo

# --------------------------------------------------------------------------

%package	core
Summary:	This package switches the GPU stack to AMDGPU PRO
Group:		System/X11

Requires:	%{libdrm_amdgpu} = %{EVRD}
Requires:	linux-firmware >= 20160816

%description	core
This package switches the GPU stack to AMDGPU PRO with the basic components.

%files	core
%dir %{_sysconfdir}/amd
%{_sysconfdir}/amd/amdapfxx.blb
%dir %{_sysconfdir}/modprobe.d
%{_sysconfdir}/modprobe.d/%{name}.conf
%dir %{_sysconfdir}/%{name}
%{_sysconfdir}/%{name}/ld.so.conf
%dir %{_datadir}/%{name}
%{_datadir}/%{name}/manifest.xml
%dir %{amdgpu_extdir}

%posttrans	core
%{_sbindir}/update-alternatives --install \
	%{_sysconfdir}/ld.so.conf.d/GL.conf gl_conf %{_sysconfdir}/%{name}/ld.so.conf %{priority} \
	--slave %{xorg_extra_modules} xorg_extra_modules %{amdgpu_extdir}

%{_sbindir}/update-alternatives --set gl_conf %{_sysconfdir}/%{name}/ld.so.conf

# The alternative ld.so.conf we have set above changed the paths where the
# system should look first for the libraries. Run ldconfig, just in case.
# The library packages or the meta-packages that pull them in should also
# do this for the system to know where to find libGL.so.1 and such.
/sbin/ldconfig -X

%preun	core
if [ $1 -eq 0 ]; then
	%{_sbindir}/update-alternatives --remove gl_conf %{_sysconfdir}/%{name}/ld.so.conf
fi

%postun	core
# We may have restored the original search paths for the libraries.
# Run ldconfig again to make sure the system will use them.
/sbin/ldconfig -X

# --------------------------------------------------------------------------

%package	libdrm-tools
Summary:	Testing tools for %{libdrm_amdgpu}
Group:		System/X11

Requires:	%{libdrm_amdgpu} = %{EVRD}
Requires:	%{libdrm} = %{EVRD}

%description	libdrm-tools
Testing tools for %{libdrm_amdgpu}.

%files	libdrm-tools
%{_bindir}/amdgpu_test
%{_bindir}/kmstest
%{_bindir}/modeprint
%{_bindir}/modetest
%{_bindir}/proptest
%{_bindir}/vbltest

# --------------------------------------------------------------------------

%package -n	x11-driver-video-%{name}
Summary:	AMDGPU PRO X11 driver
Group:		System/X11

Requires:	%{name}-core = %{EVRD}
Requires:	%{libdrm_amdgpu} = %{EVRD}
Requires:	%{libdrm} = %{EVRD}
Requires:	%{libgbm} = %{EVRD}
Requires:	%{libgl} = %{EVRD}
Requires:	xserver-abi(videodrv-%{videodrv_major})

Conflicts:	x11-driver-video-fglrx
Conflicts:	x11-driver-video-amdgpu

%description -n	x11-driver-video-%{name}
This package contains the AMDGPU PRO X11 driver.

%files -n 	x11-driver-video-%{name}
%{xorg_driverdir}/amdgpu_drv.so
%{amdgpu_extdir}/libglx.so
%{_mandir}/man4/*

# ? Readme.urpmi or the like - concerning XFdrake - to x11-driver-<...> ?
# May be but only after the support is implemented in XFdrake.

# --------------------------------------------------------------------------

%package	computing
Summary:	OpenCL components of AMDGPU PRO
Group:		System/X11

Requires:	%{name}-core = %{EVRD}
Requires:	%{name}-clinfo = %{EVRD}
Requires:	%{name}-opencl-icd = %{EVRD}
Requires:	%{libopencl} = %{EVRD}
Requires:	%{libopencl_dev} = %{EVRD}


%description	computing
This package pulls in the OpenCL components of AMDGPU PRO.

%files		computing
# No files

# --------------------------------------------------------------------------

%package	graphics
Summary:	Graphics components of AMDGPU PRO
Group:		System/X11

Requires:	%{name}-core = %{EVRD}
Requires:	%{libgl} = %{EVRD}
Requires:	%{libgles} = %{EVRD}
Requires:	%{libvdpau} = %{EVRD}
Requires:	%{libvulkan} = %{EVRD}
Requires:	%{libdri} = %{EVRD}
Requires:	x11-driver-video-%{name} = %{EVRD}


%description	graphics
This package pulls in the graphics components of AMDGPU PRO and instructs
the X11 server to use the AMDGPU PRO driver.

%files		graphics
%dir %{_sysconfdir}/amd
%{_sysconfdir}/amd/amdrc
%dir %{_sysconfdir}/gbm
%{_sysconfdir}/gbm/gbm.conf
%{xorg_confdir}/01-amdgpu-pro.conf

# ? Is it OK to package 01-amdgpu-pro.conf here?
# Should it be created by the appropriate graphics configuration tools
# (XFdrake and the like) instead?

%else
# 32-bit-only packages

%package	lib32
Summary:	32-bit AMDGPU PRO libraries
Group:		System/Libraries

Requires:	%{libgl} = %{EVRD}
Requires:	%{libgles} = %{EVRD}
Requires:	%{libvdpau} = %{EVRD}
Requires:	%{libvulkan} = %{EVRD}
Requires:	%{libdri} = %{EVRD}
Requires:	%{libgbm} = %{EVRD}
Requires:	%{name}-opencl-icd = %{EVRD}
Requires:	%{libopencl} = %{EVRD}
Requires:	%{libopencl_dev} = %{EVRD}

%description	lib32
This package pulls in the 32-bit AMDGPU PRO libraries. These can be used to
run 32-bit software on a 64-bit system with AMDGPU PRO.

%files		lib32
# No files

%endif

# --------------------------------------------------------------------------

%prep
%setup -q -n %{name}-driver

%build

# $1 - name of the package
unpack_deb() {
	pkg=$1
	mkdir -p "${pkg}"
	pushd "${pkg}"
	ar x ../${pkg}_%{pkg_ver_suffix}.deb
	tar -xf data.tar.xz
	popd
}
###############################

%ifarch x86_64
# amdgpu-pro
unpack_deb %{name}

# clinfo
unpack_deb %{name}-clinfo

# core
unpack_deb %{name}-core
sed -i 's/\/usr\/lib\/x86_64-linux-gnu\//\/usr\/lib64\//' %{name}-core/lib/%{name}/ld.conf
sed -i 's/\/usr\/lib\/i386-linux-gnu\//\/usr\/lib\//' %{name}-core/lib/%{name}/ld.conf

# libdrm-tools
unpack_deb libdrm-amdgpu-pro-tools

# x11-driver-video-amdgpu-pro
unpack_deb xserver-xorg-video-amdgpu-pro

# graphics
unpack_deb %{name}-graphics
%endif

LIBPKGS="\
	libdrm2-amdgpu-pro \
	libdrm-amdgpu-pro-amdgpu1 \
	libegl1-amdgpu-pro \
	libgbm1-amdgpu-pro \
	libgl1-amdgpu-pro-glx \
	libgles2-amdgpu-pro \
	amdgpu-pro-libopencl1 \
	amdgpu-pro-libopencl-dev \
	"

mkdir libs
for libpkg in ${LIBPKGS}; do
	unpack_deb ${libpkg}
	if test -d ${libpkg}/%{deb_libpath}/%{name}; then
		mv ${libpkg}/%{deb_libpath}/%{name}/* libs/
	fi
done

unpack_deb libgl1-amdgpu-pro-dri
unpack_deb libvdpau-amdgpu-pro

unpack_deb %{name}-opencl-icd
if test -d %{name}-opencl-icd/%{deb_libpath}/%{name}; then
	mv %{name}-opencl-icd/%{deb_libpath}/%{name}/* libs/
fi

unpack_deb %{name}-vulkan-driver
sed -i 's/\/usr\/lib\/x86_64-linux-gnu\//\/usr\/lib64\//' %{name}-vulkan-driver/etc/vulkan/icd.d/amd_icd*.json
sed -i 's/\/usr\/lib\/i386-linux-gnu\//\/usr\/lib\//' %{name}-vulkan-driver/etc/vulkan/icd.d/amd_icd*.json


%install

%ifarch x86_64
# amdgpu-pro
install -d -m 755 %{buildroot}%{_datadir}/doc/%{name}
install -m 644 %{name}%{_datadir}/doc/%{name}/copyright %{buildroot}%{_datadir}/doc/%{name}/

# clinfo
install -d -m 755 %{buildroot}%{_bindir}
mv %{name}-clinfo/usr/bin/* %{buildroot}%{_bindir}/

# core
install -d -m 755 %{buildroot}%{_sysconfdir}/amd
mv %{name}-core/etc/amd/* %{buildroot}%{_sysconfdir}/amd/
install -d -m 755 %{buildroot}%{_sysconfdir}/%{name}
mv %{name}-core/lib/%{name}/ld.conf %{buildroot}%{_sysconfdir}/%{name}/ld.so.conf
install -d -m 755 %{buildroot}%{_sysconfdir}/modprobe.d
mv %{name}-core/lib/%{name}/modprobe.conf %{buildroot}%{_sysconfdir}/modprobe.d/%{name}.conf
install -d -m 755 %{buildroot}%{_datadir}/%{name}
mv %{name}-core/usr/share/amdgpu-pro/* %{buildroot}%{_datadir}/%{name}

# libdrm-tools
mv libdrm-amdgpu-pro-tools/usr/bin/* %{buildroot}%{_bindir}/

# x11-driver-video-amdgpu-pro
install -d -m 755 %{buildroot}%{_mandir}/man4
mv xserver-xorg-video-amdgpu-pro/usr/share/man/amdgpu-pro.4/* %{buildroot}%{_mandir}/man4/
install -d -m 755 %{buildroot}%{amdgpu_extdir}
install -d -m 755 %{buildroot}%{xorg_driverdir}
mv xserver-xorg-video-amdgpu-pro/%{pkg_xorg_modulesdir}/drivers/*.so %{buildroot}%{xorg_driverdir}/
mv xserver-xorg-video-amdgpu-pro/%{pkg_xorg_modulesdir}/extensions/*.so %{buildroot}%{amdgpu_extdir}/

# graphics
install -d -m 755 %{buildroot}%{_sysconfdir}/gbm
mv %{name}-graphics/etc/amd/* %{buildroot}%{_sysconfdir}/amd
mv %{name}-graphics/etc/gbm/* %{buildroot}%{_sysconfdir}/gbm
install -d -m 755 %{buildroot}%{xorg_confdir}
mv %{name}-graphics/%{xorg_confdir}/*.conf %{buildroot}%{xorg_confdir}/
%endif

install -d -m 755 %{buildroot}%{amdgpu_libdir}
mv libs/* %{buildroot}%{amdgpu_libdir}/

install -d -m 755 %{buildroot}%{_libdir}/dri
if test -d libgl1-amdgpu-pro-dri/%{deb_libpath}/dri; then
	mv libgl1-amdgpu-pro-dri/%{deb_libpath}/dri/* %{buildroot}%{_libdir}/dri/
fi

install -d -m 755 %{buildroot}%{_libdir}/vdpau
if test -d libvdpau-amdgpu-pro/%{deb_libpath}/vdpau; then
	mv libvdpau-amdgpu-pro/%{deb_libpath}/vdpau/* %{buildroot}%{_libdir}/vdpau/
fi

############################################################################
# Some libraries from these packages (libglx.so?) may have the path to
# the DRI driver hard-coded, hence this ugly hack.
install -d -m 755 %{buildroot}/%{deb_libpath}
ln -s %{_libdir}/dri %{buildroot}/%{deb_libpath}/dri
############################################################################

install -d -m 755 %{buildroot}%{_sysconfdir}/OpenCL/vendors
mv amdgpu-pro-opencl-icd/etc/OpenCL/vendors/* %{buildroot}%{_sysconfdir}/OpenCL/vendors/

install -d -m 755 %{buildroot}%{_sysconfdir}/vulkan/icd.d
mv amdgpu-pro-vulkan-driver/etc/vulkan/icd.d/* %{buildroot}%{_sysconfdir}/vulkan/icd.d/
mv amdgpu-pro-vulkan-driver/%{deb_libpath}/* %{buildroot}%{_libdir}/

mv amdgpu-pro-libopencl-dev/%{deb_libpath}/libOpenCL.so %{buildroot}%{_libdir}/

%changelog

* Sun Nov 06 2016 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 16.30.3.315407-4.beta
- (1a80d86) Make sure the alternatives are set correctly when the packages are updated


